﻿using GameUtils.Graphics;
using GameUtils.Graphics.Data;
using Write = SharpDX.DirectWrite;

namespace GameUtils.Renderers.Direct2D1
{
    internal class Direct2DFont : FontData
    {
        Write.TextFormat font;
        readonly SharpDX.DirectWrite.Factory factory;
        string familyName;
        float size;
        FontStyle style;
        FontWeight weight;

        internal Write.TextFormat GetTextFormat()
        {
            return font;
        }

        protected override string FamilyName
        {
            get { return familyName; }
            set
            {
                familyName = value;
                font.Dispose();
                font = CreateTextFormat(factory, familyName, size, style, weight);
            }
        }

        protected override float Size
        {
            get { return size; }
            set
            {
                size = value;
                font.Dispose();
                font = CreateTextFormat(factory, familyName, size, style, weight);
            }
        }

        protected override FontStyle Style
        {
            get { return style; }
            set
            {
                style = value;
                font.Dispose();
                font = CreateTextFormat(factory, familyName, size, style, weight);
            }
        }

        protected override FontWeight Weight
        {
            get { return weight; }
            set
            {
                weight = value;
                font.Dispose();
                font = CreateTextFormat(factory, familyName, size, style, weight);
            }
        }

        private static Write.TextFormat CreateTextFormat(SharpDX.DirectWrite.Factory factory, string familyName, float size, FontStyle style, FontWeight weight)
        {
            return new Write.TextFormat(factory, familyName, Direct2DHelper.ConvertFontWeight(weight), Direct2DHelper.ConvertFontStyle(style), size);
        }

        public Direct2DFont(SharpDX.DirectWrite.Factory factory, string familyName, float size, FontStyle style, FontWeight weight)
        {
            this.factory = factory;
            this.familyName = familyName;
            this.size = size;
            this.style = style;
            this.weight = weight;
            font = CreateTextFormat(factory, familyName, size, style, weight);
        }

        protected override void Dispose(bool disposing)
        {
            font.Dispose();
        }
    }
}
