﻿using System;
using GameUtils.Graphics;
using GameUtils.Graphics.Data;
using SharpDX;
using SharpDX.Direct2D1;
using D2D = SharpDX.Direct2D1;

namespace GameUtils.Renderers.Direct2D1
{
    internal class Direct2DTextureBrush : TextureBrushData, IDirect2DBrush
    {
        readonly BitmapBrush brush;
        Texture texture;

        public Brush Brush
        {
            get { return brush; }
        }

        protected override Texture Texture
        {
            get { return texture; }
            set
            {
                Direct2DTexture data;
                try
                {
                    data = (Direct2DTexture)this.GetResourceData(value);
                }
                catch (InvalidCastException ex)
                {
                    throw new ArgumentException("Diese Texture ist nicht kompatibel.", ex);
                }

                this.texture = value;
                brush.Bitmap = data.GetBitmap();
            }
        }

        protected override TextureWrapMode WrapMode
        {
            get { return (TextureWrapMode)brush.ExtendModeX; }
            set
            {
                brush.ExtendModeX = (ExtendMode)value;
                brush.ExtendModeY = (ExtendMode)value;
            }
        }

        protected override float Opacity
        {
            get { return brush.Opacity; }
            set { brush.Opacity = value; }
        }

        internal Direct2DTextureBrush(RenderTarget target, Texture texture)
        {
            Direct2DTexture data;
            try
            {
                data = (Direct2DTexture)this.GetResourceData(texture);
            }
            catch (InvalidCastException ex)
            {
                throw new ArgumentException("Diese Texture ist nicht kompatibel.", ex);
            }

            this.texture = texture;
            brush = new BitmapBrush(target, data.GetBitmap());
        }

        protected override void SetTransform(Math.Matrix2x3 matrix)
        {
            var d2dMatrix = new Matrix3x2();
            d2dMatrix.TranslationVector = new Vector2(matrix.OffsetX, matrix.OffsetY);
            d2dMatrix.ScaleVector = new Vector2(matrix[0, 0], matrix[1, 1]);
            d2dMatrix.M12 = matrix[1, 0];
            d2dMatrix.M21 = matrix[0, 1];
            brush.Transform = d2dMatrix;
        }

        protected override void ResetTransform()
        {
            brush.Transform = Matrix3x2.Identity;
        }

        protected override void Dispose(bool disposing)
        {
            brush.Dispose();
        }
    }
}
